function transform_region_affine(im)
figure(1), imshow(im)

[imh, imw, nb] = size(im);

% get user input
disp('Choose region to copy into')
[x, y] = ginput;
mask = poly2mask(x, y, imh, imw);
[xim, yim] = meshgrid(1:imw, 1:imh);
immask = im*0.75 + 0.25*repmat(double(mask), [1 1 3]);
figure(1), imshow(immask);

disp('Select corresponding points in region to copy');
[x1, y1] = ginput;

disp('Select corresponding points in region to copy into');
[x2, y2] = ginput(numel(x1));

% compute affine transformation
A = computeAffine(x1, y1, x2, y2);
disp(num2str(A))
Ainv = inv(A);

% use transformation to copy/move region
x2m = xim(mask);
y2m = yim(mask);

pt1m = Ainv*[x2m(:)' ; 
             y2m(:)' ;
             ones(1, numel(x2m))];
x1m = pt1m(1, :);
y1m = pt1m(2, :);

im2 = im;
for b = 1:3
  imb = im2(:, :, b);
  vals = interp2(imb,x1m,y1m, 'linear');
  imb(mask) = vals;
  %imb(mask) = imb(round(y1m) + round(x1m-1)*imh);
  im2(:, :, b) = imb;
end
figure(2), imshow(im2);



function aff = computeAffine(x1, y1, x2, y2)
% [x2 y2 1] = aff*[x1 y1 1]' 
% A x = b where x = [a b c d e f]'
% x2 = ax1 + by1 + c --> [x1 y1 1  0  0 0] x = x2 
% y2 = dx1 + ey1 + f --> [0  0  0 x1 y1 1] x = y2

A = zeros(6, 6);
b = zeros(6, 1);
c = 0;
for k = 1:3
  c = c+1;
  A(c, :) = [x1(k) y1(k) 1 0 0 0];
  b(c) = x2(k);
  c = c+1;
  A(c, :) = [0 0 0 x1(k) y1(k) 1];
  b(c) = y2(k);  
end

params = A\b;
aff = zeros(3);
aff(1, :) = params(1:3);
aff(2, :) = params(4:6);
aff(3,3) = 1;